@@ -36,8 +36,19 @@ from gevent.event import AsyncResult
 
 from logging import root as logger, currentframe
 
-DEBUG = 1
+DEBUG = 0
 PICKLE_PROTOCOL = HIGHEST_PROTOCOL
+
+def log1():
+    from gevent import socket
+    old_read = socket.wait_read
+    def _wait_read(*args, **kw):
+        if DEBUG:
+            s = traceback.format_stack()
+            printf(s)
+        return old_read(*args, **kw)
+    socket.wait_read = _wait_read
+
 class RpcError(StandardError):
     pass
 
@@ -107,7 +118,7 @@ def printf(msg, *args):
 def log_except(*args, **kw):
     ei = sys.exc_info()
     if args:
-        lines = [agrs[0] % args[1:], '\n']
+        lines = [args[0] % args[1:], '\n']
     else:
         lines = []
     lines.append('Traceback (most recent call last):\n')
@@ -262,6 +273,8 @@ class RpcBase(object):
         except socket_error:
             pass
         self._socket.close()
+        self._socket = None
+        #_rpcs_.append('-%s' % str(self.addr))
 
     def after_start(self):
         pass
@@ -271,6 +284,7 @@ class RpcBase(object):
             return
         self.stoped = False
         self.after_start()
+        #_rpcs_.append(self.addr)
 
     def before_stop(self):
         pass
@@ -331,6 +345,10 @@ class RpcService(object):
         self.svr = svr
         #self._pool = Pool(size=size)
         self.sock = sock
+        if isinstance(svr, RpcClient):
+            self.sock_addr = svr.addr
+        else:
+            self.sock_addr = self.sock.getpeername()
         self.uid = str(uid)
         if len(self.uid) != self.UID_LEN:
             raise ValueError, 'uid length error: len(uid)=%d <> %d' % (len(uid), self.UID_LEN)
@@ -352,24 +370,35 @@ class RpcService(object):
         if not self.stoped:
             return
         self.stoped = False
-##        self._send_queue = Queue(maxsize=0)
-##        self._send_task = spawn(self._sender)
         self._recv_task = spawn(self._recver)
         self._recver_on_error = False
+        #_services_.append(self.sock_addr)
 
     def remote_stop(self):
+        #printf('remote_stop:%s', self.sock_addr)
         self.sock_error = True
-        spawn(self.stop)
+        self.stop()
+
+    def close(self):
+        if not self.sock:
+            return
+        try:
+            self.sock._sock.close()
+        except socket_error:
+            pass
+        self.sock.close()
+        self.sock = None
 
     def stop(self):
         if self.stoped:
             return
         self.stoped = True
+        self._recv_task.kill(block=True)
+        self._recv_task = None
         self.svr.svc_stop(self)
-        self._stop_resps()
-        self._stop_proxys()
-        if not self.sock_error:
+        if 1 and not self.sock_error:
             try:
+                #printf('remote_stop:%s', self.sock_addr)
                 self.call(0, 'remote_stop', tuple(), None, no_result=True)
                 sleep(0.05)
             except:
@@ -377,15 +406,12 @@ class RpcService(object):
         if getattr(self, '_heart_task', None):
             self._heart_task.kill(block=False)
             self._heart_task = None
-
         try:
-            pass #self._pool.join(timeout=60*3, raise_error=True)
+            self._stop_resps()
+            self._stop_proxys()
         finally:
-            self.sock.close()
-##            self._send_task.kill(block=False)
-##            self._send_task = None
-            self._recv_task.kill(block=False)
-            self._recv_task = None
+            self.close()
+            #_services_.append('-%s' % str(self.sock_addr))
 
     def _stop_resps(self):
         error = RpcRuntimeError('service stoped')
@@ -413,36 +439,38 @@ class RpcService(object):
 
     def _recver(self):
         """ 接收处理数据 """
+        def _read(c):
+            d = self.sock.recv(c)
+            if not d:
+                if self.stoped:
+                    raise GreenletExit
+                self._recver_on_error = True
+                self._on_socket_error(None)
+                self._recver_on_error = False
+                return None
+            return d
+
         try:
             sio = StringIO()
             while not self.stoped:
-                try:
-                    d = self.sock.recv(4)
-                    if not d:
-                        if self.stoped:
-                            break
-                        self._recver_on_error = True
-                        self._on_socket_error(None)
-                        self._recver_on_error = False
+                d = _read(4)
+                if d is None:
+                    continue
+                dlen = unpack('I', d)[0]
+                #rs = []
+                sio.seek(0)
+                sio.truncate()
+                while dlen > 0:
+                    data = _read(dlen)
+                    if data is None:
                         continue
-                    dlen = unpack('I', d)[0]
-                    #rs = []
-                    sio.seek(0)
-                    sio.truncate()
-                    while dlen > 0:
-                        data = self.sock.recv(dlen)
-                        #rs.append(data)
-                        sio.write(data)
-                        dlen -= len(data)
-                    #spawn(self._handle, loads(''.join(rs)))
-                    sio.seek(0)
-                    spawn(self._handle, load(sio))
-                    #self._pool.spawn(self._handle, loads(''.join(rs)))
-                except socket_error as err:
-                    self._recver_on_error = True
-                    self._on_socket_error(err)
-                    self._recver_on_error = False
-
+                    #rs.append(data)
+                    sio.write(data)
+                    dlen -= len(data)
+                #spawn(self._handle, loads(''.join(rs)))
+                sio.seek(0)
+                spawn(self._handle, load(sio))
+                #self._pool.spawn(self._handle, loads(''.join(rs)))
         except GreenletExit:
             pass
         except Exception as err:
@@ -801,7 +829,7 @@ class RpcServer(RpcBase):
         self._services.pop(service.uid, None)
 
     def _accept(self):
-        while 1:
+        while not self.stoped:
             try:
                 client_socket, address = self._socket.accept()
                 def _accept_sock(client_socket, address):
@@ -838,7 +866,7 @@ class RpcProxy(object):
         if addr:
             self._addr = addr
         else:
-            self._addr = self._svc.sock.getpeername()
+            self._addr = self._svc.sock_addr
         self.timeout = CALL_TIMEORUT
         self._closes = {}#WeakKeyDictionary()
 
@@ -1003,6 +1031,8 @@ def map_items_ex(proxys, attr, *args, **kw):
 
 _my_servers_ = WeakValueDictionary()
 _addr_clients = WeakValueDictionary()
+_rpcs_ = []
+_services_ = []
 def uninit():
     """ clear all """
     global _my_servers_, _addr_clients
@@ -1014,6 +1044,7 @@ def uninit():
         cli.stop()
     for svr in servers:
         svr.stop()
+    #printf('_rpcs_:%s \n_services_:%s', _rpcs_, _services_)
 
 def get_rpc_by_addr(addr, force_rpc=0):
     global _addr_clients, _my_servers_
@@ -1036,6 +1067,8 @@ def get_rpc_by_addr(addr, force_rpc=0):
         return client
 
 def get_proxy_by_addr(addr, proxy_name, proxy_cls=None):
+#    if proxy_name == 'game':
+#        printf('get_proxy_by_addr:%s', addr)
     rpc_obj = get_rpc_by_addr(addr)
     if rpc_obj is None:
         return
